﻿using System;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace TestoInImmagine
{
    public partial class Form1 : Form
    {
        string currentImagePath = "";
        const string marker = "--97725318--"; // marker che notifica l'aggiunta di un testo
        byte[] cleanImageBytes; // immagine pulita, senza testo

        public Form1()
        {
            InitializeComponent();
        }

        private void btnOpenImage_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog ofd = new OpenFileDialog())
            {
                ofd.Filter = "Immagini|*.jpg;*.jpeg;*.png;*.bmp";
                if (ofd.ShowDialog() == DialogResult.OK)
                {
                    currentImagePath = ofd.FileName;
                    byte[] imageData = File.ReadAllBytes(currentImagePath); // in imageData tutti i byte dell'immagine aperta
                    using (var ms = new MemoryStream(imageData)) // copia dell'immagine in memoria
                    {
                        pictureBox1.Image = Image.FromStream(ms); // visualizzazione con la copia, evita che il file originale resti in uso.
                    }
                    string hidden = ExtractText(imageData); // ricerca di un testo accodato
                    textBoxHiddenText.Text = hidden ?? ""; // se il testo c'è viene visualizzato altrimenti textbox vuota
                }
            }
        }

        private void btnSaveImage_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(currentImagePath)) return; // ritorna, se non esiste un percorso di salvataggio

            using (SaveFileDialog sfd = new SaveFileDialog())
            {
                string originalExtension = Path.GetExtension(currentImagePath); // l'estenzione non verrà modificata
                sfd.Filter = $"Image file (*{originalExtension})|*{originalExtension}";
                sfd.DefaultExt = originalExtension;
                sfd.FileName = Path.GetFileName(currentImagePath); // proposta nomefile invariato

                if (sfd.ShowDialog() == DialogResult.OK)
                {
                    EmbedText(currentImagePath, sfd.FileName, textBoxHiddenText.Text); // aggiunta del testo
                    MessageBox.Show("Salvato con successo.");
                }
            }
        }

        void EmbedText(string imagePath, string outputPath, string text)
        {
            byte[] textBytes = Encoding.UTF8.GetBytes(marker + text);
            byte[] combinedBytes = new byte[cleanImageBytes.Length + textBytes.Length]; // dimensionamento array
            Buffer.BlockCopy(cleanImageBytes, 0, combinedBytes, 0, cleanImageBytes.Length); // aggiunta dei byte che rappresentano l'immagine
            Buffer.BlockCopy(textBytes, 0, combinedBytes, cleanImageBytes.Length, textBytes.Length); // aggiunta dei byte che rappresentano il testo
            File.WriteAllBytes(outputPath, combinedBytes); // salvataggio
        }

        string ExtractText(byte[] fileBytes)
        {
            byte[] markerBytes = Encoding.UTF8.GetBytes(marker);
            int index = IndexOfSequence(fileBytes, markerBytes); // Cerca il marker nei byte

            if (index >= 0)
            {
                cleanImageBytes = fileBytes.Take(index).ToArray(); // l'immagine contiene già un testo, in cleanImageBytes viene eliminato
                byte[] hiddenTextBytes = fileBytes.Skip(index + markerBytes.Length).ToArray(); // il testo
                return Encoding.UTF8.GetString(hiddenTextBytes); // Testo in UTF8
            }
            else
            {
                cleanImageBytes = fileBytes.ToArray(); // l'immagine non conteneva un testo, in cleanImageBytes l'immagine originale
                return null; // nessun testo rilevato
            }
        }

        int IndexOfSequence(byte[] buffer, byte[] pattern) // ricerca byte per byte del marker
        {
            for (int i = 0; i < buffer.Length - pattern.Length; i++)
            {
                bool match = true;
                for (int j = 0; j < pattern.Length; j++)
                {
                    if (buffer[i + j] != pattern[j])
                    {
                        match = false;
                        break;
                    }
                }
                if (match) return i; // il marker è stato trovato, restituisco la posizione
            }
            return -1; // il marker non è stato trovato, restituisco -1
        }
    }
}
